//
//  SwiftCloverConnectPluggin.swift
//  clover_connect
//
//  Created by Phan.dev
//

import Flutter
import UIKit
import CloverConnector

public class SwiftCloverConnectPlugin: NSObject, FlutterPlugin {
    var fCloverConnect: CloverConnect!
    
    public static func register(with registrar: FlutterPluginRegistrar) {
        let channel = FlutterMethodChannel(name: "phan.dev/clover_connect", binaryMessenger: registrar.messenger())
        let instance = SwiftCloverConnectPlugin()
        registrar.addMethodCallDelegate(instance, channel: channel)
    }
    
    public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        switch call.method {
        case Method.connect:
            let connectionConfig = call.arguments as! [String: String]
            let response = connect(connectionConfig: connectionConfig)
            result(response)
            
        case Method.reset:
            let response = reset()
            result(response)
            
        case Method.disconnect:
            let response = disconnect()
            result(response)
            
        case Method.getLastEvent:
            result(fCloverConnect.lastEvent)
            
        case Method.getPairingCode:
            result(fCloverConnect.pairingCode)
            
        case Method.getDeviceEvent:
            result(fCloverConnect.deviceEvent)
            
        case Method.sale:
            let saleRequest = call.arguments as! [String: Any]
            let response = sale(saleRequest: saleRequest)
            result(response)
            
        default:
            result(FlutterMethodNotImplemented)
        }
    }
    
    private func connect(connectionConfig: [String: String]) -> String {
        var fCloverConnector: ICloverConnector?
        
        fCloverConnect = CloverConnect(cloverConnector: fCloverConnector)
        fCloverConnect.connect(connectionConfig: connectionConfig)
        
        return ResultCode.SUCCESS
    }
    
    private func reset() -> String {
        fCloverConnect.reset()
        
        return ResultCode.SUCCESS
    }
    
    private func disconnect() -> String {
        fCloverConnect.disconnect()
        
        return ResultCode.SUCCESS
    }
    
    private func sale(saleRequest: [String: Any]) -> String {
        fCloverConnect.doSale(saleRequest: saleRequest)
        
        return ResultCode.SUCCESS
    }
}
