import 'dart:async';
import 'dart:convert';

import 'package:flutter/services.dart';

/// Main class
class CloverConnect {
  static const MethodChannel _channel =
      const MethodChannel('phan.dev/clover_connect');

  static const _methodConnect = 'connect';
  static const _methodDisconnect = 'disconnect';
  static const _methodReset = 'reset';
  static const _methodSale = 'sale';
  static const _methodGetLastEvent = 'getLastEvent';
  static const _methodGetPairingCode = 'getPairingCode';
  static const _methodGetDeviceEvent = 'getDeviceEvent';

  /// Connects to Clover using [cloverConfig]
  static Future<String> connect(CloverConfig cloverConfig) async {
    final String response =
        await _channel.invokeMethod(_methodConnect, cloverConfig.toMap());
    return response;
  }

  /// Resets
  static Future<String> reset() async {
    final String response = await _channel.invokeMethod(_methodReset);
    return response;
  }

  /// Disconnects
  static Future<String> disconnect() async {
    final String response = await _channel.invokeMethod(_methodDisconnect);
    return response;
  }

  /// Gets pairing code
  static Future<String> get pairingCode async {
    final String response = await _channel.invokeMethod(_methodGetPairingCode);
    return response;
  }

  /// Gets last event
  static Future<String> get lastEvent async {
    final String response = await _channel.invokeMethod(_methodGetLastEvent);
    return response;
  }

  /// Gets device event
  static Future<Map> get deviceEvent async {
    final String response = await _channel.invokeMethod(_methodGetDeviceEvent);
    final Map deviceEventMap = jsonDecode(response);
    return deviceEventMap;
  }

  /// Does sale using [saleRequest]
  static Future<String> sale(SaleRequest saleRequest) async {
    final String response =
        await _channel.invokeMethod(_methodSale, saleRequest.toMap());
    return response;
  }
}

/// [CloverConfig]
class CloverConfig {
  /// Default constructor
  CloverConfig(
      {this.endpoint = 'wss://192.168.1.123:12345/remote_pay',
      this.remoteApplicationID = 'dev.phan.flutter.clover_connect',
      this.posName = 'Flutter POS',
      this.posSerial = 'POS-123'});

  ///
  String endpoint, remoteApplicationID, posName, posSerial;

  /// Convert [CloverConfig] to Map<String, String>
  Map<String, String> toMap() => {
        'endpoint': endpoint,
        'remoteApplicationID': remoteApplicationID,
        'posName': posName,
        'posSerial': posSerial,
      };
}

/// [SaleRequest]
class SaleRequest {
  /// Default constructor
  SaleRequest({
    this.amount = 0,
    this.externalId = "",
  });

  ///
  int amount;

  ///
  String externalId;

  /// Convert [SaleRequest] to Map<String, dynamic>
  Map<String, dynamic> toMap() => {
        'amount': amount,
        'externalId': externalId,
      };
}
